#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'extracter/do_extract_what_to.rb'
# =========================================================================== #
module Extracter

class Extracter

  # ========================================================================= #
  # === do_extract_what_to
  #
  # This method should only be called from the method
  # work_on_the_given_input().
  #
  # It will attempt to extract the specified archive.
  # ========================================================================= #
  def do_extract_what_to(
      what, # ← The archive or file that we wish to extract.
      to = extract_to?
    )
    name_of_the_archive = what.dup
    @did_we_extract_already = false
    to = extract_to? if to.nil?
    set_source_location(what)
    set_extract_to_this_location(to) # Keep it in sync.
    # ======================================================================= #
    # Create the directory if it does not yet exist.
    # ======================================================================= #
    create_directory(to) if !File.directory?(to)
    if be_verbose?
      if @show_only_the_short_name_of_the_archive
        name_of_the_archive = File.basename(name_of_the_archive)
        copn; e "Extracting `#{sfancy(name_of_the_archive)}` to `#{sdir(to)}` next."
      else
        copn; e "Extracting `#{sfancy(name_of_the_archive)}` to `#{sdir(to)}` next."
      end
    end
    _ = ''.dup # Default.
    extname = File.extname(what)
    unless ARRAY_REGISTERED_ARCHIVES.include? extname.delete('.')
      opn; e 'We did not register the following extension: '+extname.delete('.')
      fail_message_not_registered(extname)
      @skip_extracting = true
    end
    case extname # Case tag. Those listed on top are more important.
    # ======================================================================= #
    # === .tar
    # ======================================================================= #
    when '.tar',
         '.tar.bz2',
         '.tbz'
      _ << 'tar -xvf'
    # ======================================================================= #
    # === pdf
    #
    # For pdf-files we will tap into the pdf_paradise project.
    # ======================================================================= #
    when /\.?pdf$/
      begin
        require 'pdf_paradise/convert_pdf_to_text.rb'
        _ = PdfParadise::ConvertPdfToText.new(what)
        _.output_file?
      rescue LoadError; end
      return # Must return early in this case.
    # ======================================================================= #
    # === img
    #
    # Note that .img in this context refers to squafhs .img files.
    # ======================================================================= #
    when '.img',
         '.squashfs'
      opnn; e 'Handling a squashfs .img file format next:'
      name_without_extension = what.sub(/#{File.extname(what)}$/,'')
      mkdir(name_without_extension) unless File.directory? name_without_extension
      esystem 'mount -o loop -t squashfs '+what+' '+name_without_extension
      e 'The content of the extracted (or rather, mounted) archive is:'
      pp Dir["#{name_without_extension}*"]
      return # Must return early in this case.
    # ======================================================================= #
    # === iso
    # ======================================================================= #
    when '.iso'
      opnn; e 'Extracting an .iso file is a bit more complicated '\
              'than a .tar.gz tarball release.'
      opnn; e 'We will first create a directory; and then mount '\
              'the .iso there.'
      name_without_extension = what.sub(/#{File.extname(what)}$/,'')
      mkdir(name_without_extension) unless File.directory? name_without_extension
      esystem 'mount -o loop '+what+' '+name_without_extension
      e 'The content of the extracted (or rather, mounted) archive is:'
      pp Dir[name_without_extension+'*']
      return
    # ======================================================================= #
    # === sxz
    # ======================================================================= #
    when '.sxz'
      _ = 'unsquashfs '.dup # This requires squashfs with xz-support.
    # ======================================================================= #
    # === lz
    # ======================================================================= #
    when '.lz','.tar.lz'
      _ = 'tar --lzip -xvf '.dup # This requires lzip to be installed.
    # ======================================================================= #
    # === tar.Z
    # ======================================================================= #
    when '.tar.Z','.taz'
      _ << 'tar -xvzf'
    # ======================================================================= #
    # === .jar
    # ======================================================================= #
    when /\.?jar$/i
      _ << 'jar xvf'
    # ======================================================================= #
    # === .zst
    #
    # This entry point is for e. g. "pymol-2.3.0-3-x86_64.pkg.tar.zst".
    # ======================================================================= #
    when '.zst','.tar.zst'
      _ << 'tar -I zstd -xvf '
    # ======================================================================= #
    # === txz
    # ======================================================================= #
    when '.txz'
      _ << 'tar Jxvf' # Since Jan 2012.
    # ======================================================================= #
    # === gz
    # ======================================================================= #
    when '.gz'
      if what.include? '.tar'
        _ << 'tar -zxvf'
      else
        _ << 'gunzip'
      end
    # ======================================================================= #
    # === xz
    # ======================================================================= #
    when '.xz'
      if _.include? '.tar'
      end
      _ << 'tar -xvf' # tar -Jxv #{what} would be an alternative.
    # ======================================================================= #
    # === rpm
    # ======================================================================= #
    when '.rpm'
      _ << 'bsdtar xfv'
    # ======================================================================= #
    # === bin
    # ======================================================================= #
    when '.bin' # handle .bin files here.
      # _ = 'tar -jxf '+package
      _ << "./#{what}"
    # ======================================================================= #
    # === zip
    # ======================================================================= #
    when '.zip','.xpi','.docx','.odt', # .docx and .odt format types can be unpacked with zip too.
         '.apkg'
      # _ << 'ar -jxf' # unzip #{what} <-- this should work as well.
      _ << 'unzip '
    when /\.bz2/,'.tbz2'
      if what.include? '.tar' # Treat it as a .tar file.
        _ << 'tar -vjxf '
      else
        _ << 'bunzip2 '
      end
    # ======================================================================= #
    # === lzma
    # ======================================================================= #
    when '.lzma'
      _ << 'unlzma '
    # ======================================================================= #
    # === 7z
    # ======================================================================= #
    when '.7z' # 7z does not accept the -C commandline.
      # _ << '7za e' # <- Deprecated as of 05.06.2020.
      _ << '7z x'
    # ======================================================================= #
    # === gem
    # ======================================================================= #
    when '.gem'
      _ << GEM_UNPACK_COMMAND
    # ======================================================================= #
    # === rar
    # ======================================================================= #
    when '.rar'
      check_whether_rar_is_available
      _ << 'unrar e'
    # ======================================================================= #
    # === deb
    # ======================================================================= #
    when '.deb'
      #_ = 'dpkg-deb -x' # {to}
      _ << 'ar -x' # ar -x #{what} This could work too.
    # ======================================================================= #
    # === tgz
    # ======================================================================= #
    when '.tgz'
      _ << 'tar -xvzf'
    # ======================================================================= #
    # === ps
    # ======================================================================= #
    when '.ps'
      _ << 'ps2ascii'
    # ======================================================================= #
    # === mp4
    # ======================================================================= #
    when '.mp4' # If it is a .mp4 file, we delegate to ExtractAudio instead.
      if Object.const_defined? :MultimediaParadise
        MultimediaParadise.extract_audio(@source_package_location)
      end
      exit
    else # else tag. We did not find the extension type.
      @skip_extracting = true
      copn; ewarn "We did not find: `#{sfile(what)}`. "
      copn; ewarn 'The file-type (extension) was: `'+simp(extname)+'`.'
      # Try to rescue though.
      result = run_this_system_command("file #{what}")
      copn; e result
      if result.include? 'bzip2 compressed'
        copn; e 'We assume it is a .bz2 file though.'
        _ = 'tar -vjxf '.dup
        @skip_extracting = false
      end
    end unless @skip_extracting
    unless File.exist? what
      @skip_extracting = true
      copn; e "The file `#{sfile(what)}"\
              "` does not exist - can not extract."
    end
    # ======================================================================= #
    # Handle the situation when the given input includes a ')' character.
    # We will pad such an input with '"' characters.
    # ======================================================================= #
    if what.include? ')'
      what = pad(what, '"') #sanitize_input(what)
    end
    # ======================================================================= #
    # Next, pad it if it includes a ' ' character.
    # ======================================================================= #
    what = pad(what) if what.include?(' ')
    _ << " #{what}" unless _.empty?
    if _.include? GEM_UNPACK_COMMAND # Gem command needs a --target=DIR option.
      _ << " --target=#{to}"
    elsif _.include?('ar -x') and ! _.include?('.tar') # Do nothing in this case.
    elsif _.end_with? '.sxz'
      # .sxz does not require the -C option.
    elsif _.end_with? '.zip','.xpi','.7z','.jar','.apkg'
      # Do not use -C option for 7z, as it hates this option.
      # .jar files also do not support the -C option.
    elsif _.include? 'bunzip'
      # Do not use -C option for bunzip, as it hates this option.
    else
      # ===================================================================== #
      # Next, we need to determine the location where we extract our
      # archive to.
      # ===================================================================== #
      _ << ' '
      # ===================================================================== #
      # Add -C option except for .deb packages and gunzip-based archives.
      # ===================================================================== #
      unless _ =~ /deb$/ or _.include?('gunzip')
        _ << '-C '
        _ << to
      end
    end
    if run_simulation?
      copn; e 'As we are running in simulation mode, the following command '
      copn; e 'is the one that we would have been used if we were to not run '
      copn; e 'in simulation mode:'
      e _
    else # Ok, here we are not in a simulation, hence we can run the command.
      unless @skip_extracting
        if File.writable? to
          # ================================================================= #
          # Next, run the sys-command, with some padding.
          # ================================================================= #
          begin
            run_this_system_command(
              "  #{_}", :also_show_what_we_will_do
            )
          # ================================================================= #
          # We have to rescue because unrar might not be available and so on.
          # ================================================================= #
          rescue Exception => error
            e 'An error has happened upon attempting to run this system command:'
            e
            e "  #{_}"
            e
            pp error
            e '-----------'
            pp error.class
            e '-----------'
          end
          @did_we_extract_already = true
        else
          copn; ewarn 'You do not have sufficient permissions to'
          copn; ewarn "modify #{sdir(to)}."
        end
      end
    end
  end; alias do_extract_to do_extract_what_to # === do_extract_to
       alias start         do_extract_what_to # === start

end; end