#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Extracter::ExtractIt
#
# If no input was provided to this class, but a .zip file exists in
# the current working directory, then that .zip file will be used.
#
# Usage example:
#
#   Extracter::ExtractIt.new(ARGV)
#
# =========================================================================== #
# require 'extracter/extract_it/extract_it.rb'
# =========================================================================== #
module Extracter

class ExtractIt # === Extracter::ExtractIt

  begin
    require 'opn'
  rescue LoadError; end

  begin
    require 'remove_file_suffix'
  rescue LoadError; end

  begin
    require 'colours'
    include Colours if Object.const_defined? :Colours
  rescue LoadError; end

  require 'extracter/extracter.rb'

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === ExtractIt::ARRAY_ARCHIVE_TYPES
  #
  # Register the available (and handled) archive types here.
  # ========================================================================= #
  ARRAY_ARCHIVE_TYPES = %w(
    .apkg
    .xz
    .zip
    .tar
    .bz2
    .7z
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_set_input = nil,
      run_already        = true
    )
    reset
    set_input(optional_set_input)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    @debug = false
  end

  # ========================================================================= #
  # === set_input
  #
  # We will work on an Array as value to @input, at the end of this
  # method.
  # ========================================================================= #
  def set_input(i = N)
    case i
    # ======================================================================= #
    # === extract_it --help
    # ======================================================================= #
    when /-?-?help$/i # Show some help stuff here.
      e 'We will show a little bit help, then exit.'
      e
      e 'To extract .tar.xz, do:'
      e
      efancy '  -> tar -xJf *.tar.xz'
      exit
    end
    i = [i] if i.is_a? String # Need an Array.
    if @debug
      opn; e 'The input given to us is: `'+sfile(i)+'`'
    end
    if i.is_a?(Array) and i.empty?
      # ===================================================================== #
      # In this case, try to see if the current directory has a .zip
      # file. We will use this in that case.
      # ===================================================================== #
      is_there_a_zip_file = Dir['*.zip']
      unless is_there_a_zip_file.empty?
        use_this_zip_file = is_there_a_zip_file.first
        notify_the_user_that_no_input_was_given_but_this_file_was_found(use_this_zip_file)
        i << use_this_zip_file
      end
      is_there_at_the_least_one_tar_xz_file = Dir['*.tar.xz']
      unless is_there_at_the_least_one_tar_xz_file.empty?
        i << is_there_at_the_least_one_tar_xz_file.first
      end
    end
    @input = i # Should be an array, always.
  end

  # ========================================================================= #
  # === be_silent
  # ========================================================================= #
  def be_silent
    @be_silent = true
  end

  # ========================================================================= #
  # === be_verbose
  # ========================================================================= #
  def be_verbose
    @be_silent = false
  end; alias show_commands_used be_verbose # === show_commands_used

  # ========================================================================= #
  # === notify_the_user_that_no_input_was_given_but_this_file_was_found
  # ========================================================================= #
  def notify_the_user_that_no_input_was_given_but_this_file_was_found(
      this_file
    )
    opn; e 'No input was given to '+sfancy(NAMESPACE)+' but a '\
           '.zip file was'
    opn; e 'found in this directory ('+sdir(Dir.pwd)+'): '+
            sfancy(this_file)
    opn; e 'We will use this zip file.'
  end

  # ========================================================================= #
  # === extract_input
  # ========================================================================= #
  def extract_input
    pp @input if @debug
    @input.each {|entry|
      to_this_dir = Dir.pwd
      to_this_dir << '/' unless to_this_dir.end_with? '/'
      unless File.exist? entry
        entry = try_to_glob_on(entry)
      end
      # ===================================================================== #
      # Delegate to class Extracter next.
      # ===================================================================== #
      Extracter.extract_what_to(
        entry,
        to_this_dir,
        @be_silent
      )
      _ = RemoveFileSuffix[entry]
      if File.exist? entry
        # =================================================================== #
        # Must also check whether the extracted directory exists.
        # =================================================================== #
        name_of_the_extracted_archive = to_this_dir+_
        ARRAY_ARCHIVE_TYPES.each {|extension_name|
          if name_of_the_extracted_archive.include? extension_name
            quoted = Regexp.quote(extension_name)
            name_of_the_extracted_archive.sub!(/#{quoted}$/,'')
          end
        }
        if File.exist?(name_of_the_extracted_archive) and
          # ================================================================= #
          # The following check ensures that we really have another name
          # for the extracted directory.
          # ================================================================= #
          !(entry == name_of_the_extracted_archive)
          opn; e "Finished extracting #{sfile(_)} to `#{sdir(to_this_dir)}`."
        else
          opn; e "No file called `#{sfile(name_of_the_extracted_archive)}"\
                 "` appears to exist."
        end
      end
    }
  end

  # ========================================================================= #
  # === is_archive?
  # ========================================================================= #
  def is_archive?(i)
    ARRAY_ARCHIVE_TYPES.include?(
      File.extname(File.basename(i))
    )
  end

  # ========================================================================= #
  # === try_to_glob_on
  # ========================================================================= #
  def try_to_glob_on(i)
    result = Dir["#{i}*"]
    # ======================================================================= #
    # Next, sort this result to put archives on the beginning of the Array.
    # ======================================================================= #
    result = result.partition {|entry| is_archive?(entry) }
    result.flatten!
    unless result.empty?
      # ===================================================================== #
      # Ok, we grab the first entry next.
      # ===================================================================== #
      i = result.first
      opn; e "No result could be found for the given input, thus "\
             "using #{sfancy(i)} instead."
    end
    i
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    extract_input
  end

  # ========================================================================= #
  # === Extracter::ExtractIt[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Extracter::ExtractIt.new(ARGV)
end # extract_it.rb